/****************************************************************
 *        Copyright   2005 A. Farina All Rights Reserved       *
 ****************************************************************/

#include <windows.h>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

                                                      

void IIR(float  *fIRFilt,long IRL,double Alfa,double Beta,double Gamma,double Mu,double Sigma)
{
	// Filtraggio IIR a 2 poli (Bi-Quad), il risultato sovrascrive il vettore di ingresso
	long n=0;
	double Xn;
	double Xnm1=0.0;
	double Xnm2=0.0;
	double Ynm1=0.0;
	double Ynm2=0.0;

	Xn = fIRFilt[n];
	// X(n) = 2 * (Alfa * (Xn + Mu * Xnm1 + Sigma * Xnm2) + Gamma * Ynm1 - Beta * Ynm2) - Basic
	fIRFilt[n]=(float)(2.0*(Alfa*(Xn+Mu*Xnm1+Sigma*Xnm2)+Gamma*Ynm1-Beta*Ynm2));
	Xnm2 = Xnm1;
	Xnm1 = Xn;

	n=1;
	Xn = fIRFilt[n];
	Ynm1=fIRFilt[n-1];
	fIRFilt[n]=(float)(2.0*(Alfa*(Xn+Mu*Xnm1+Sigma*Xnm2)+Gamma*Ynm1-Beta*Ynm2));
	Xnm2 = Xnm1;
	Xnm1 = Xn;
	
	for(n=2;n<IRL;n++)
	{
		Xn = fIRFilt[n];
		Ynm1 = fIRFilt[n-1];
		Ynm2 = fIRFilt[n-2];
		fIRFilt[n]=(float)(2.0*(Alfa*(Xn+Mu*Xnm1+Sigma*Xnm2)+Gamma*Ynm1-Beta*Ynm2));
		Xnm2 = Xnm1;
		Xnm1 = Xn;
	}
}

void Gain(float dBgain, float  *X,long IRL)
{
	long k;
	float A=(float)pow(10.0,dBgain/20);
	for (k=1; k<IRL;k++) X[k]*=A;
}

void AccFilter(float  *X,float fSamp,long IRL)
{   
	// Ponderazione Accelerazione Asse Generico secondo norma ISO-8041
	
	double PI=3.141592653589793;
	
// filtro 1 - passa alto 0.7943 Hz
	double coef1 =  1.0 - exp(-2.0 * PI * (0.7943) / fSamp);
	double coef1m = 1.0 - coef1;
// filtro 2 - passa alto 0.7943 Hz
	double coef2 =  1.0 - exp(-2.0 * PI * (0.7943) / fSamp);
	double coef2m = 1.0 - coef2;
// filtro 3 - passa basso 5.684 Hz
	double coef3 =  1.0 - exp(-2.0 * PI * (5.684) / fSamp);
	double coef3m = 1.0 - coef3;
// filtro 4 - passa basso 100 Hz
	double coef4 =  1.0 - exp(-2.0 * PI * (100.0) / fSamp);
	double coef4m = 1.0 - coef4;
// filtro 5 - passa basso 100 Hz
	double coef5 =  1.0 - exp(-2.0 * PI * (100.0) / fSamp);
	double coef5m = 1.0 - coef5;

	long k;
	double ritardo1=0.0,ritardo2=0.0;
	double A,B,C=0.0,D=0.0,E=0.0;

	for (k=0; k<IRL;k++)
	{
//		primi 2 filtri passa alto in cascata
		A = X[k] - ritardo1;
		ritardo1 = X[k] * coef1 + ritardo1 * coef1m;
		B = A - ritardo2;
		ritardo2 = A * coef2 + ritardo2 * coef2m;

//		3 filtri passa basso conclusivi
		C = B * coef3 + C * coef3m;
		D = C * coef4 + D * coef4m;
		E = D * coef5 + E * coef5m;
		X[k]=(float)(E*1.19); // correzione per rendere il guadagno ad 1 Hz = 0 dB
	}
}


void AFilter(float  *X,float fSamp,long IRL)
{   
	// Ponderazione "A" secondo norma IEC
	// filtro IIR a 2 poli
	double PI=3.141592653589793;
//  filtro 1 - passa alto 20.6 Hz
	double coef1 =  1.0 - exp(-2.0 * PI * (20.6) / fSamp);
	double coef1m = 1.0 - coef1;
//  filtro 2 - passa alto 20.6 Hz
	double coef2 =  1.0 - exp(-2.0 * PI * (20.6) / fSamp);
	double coef2m = 1.0 - coef2;
//  filtro 3 - passa alto 107.7 Hz
	double coef3 =  1.0 - exp(-2.0 * PI * (107.7) / fSamp);
	double coef3m = 1.0 - coef3;
//  filtro 4 - passa.0 alto 737.9 Hz
	double coef4 =  1.0 - exp(-2.0 * PI * (737.9) / fSamp);
	double coef4m = 1.0 - coef4;
//  filtro 5 - passa basso 12200 Hz
	double coef5 =  1.0 - exp(-2.0 * PI * (12200.0) / fSamp);
	double coef5m = 1.0 - coef5;
//  filtro 6 - passa basso 12200 Hz
	double coef6 =  1.0 - exp(-2.0 * PI * (12200.0) / fSamp);
	double coef6m = 1.0 - coef6;
	long k;
	double ritardo1=0.0,ritardo2=0.0,ritardo3=0.0,ritardo4=0.0;
	double A,B,C,D,E=0.0,F=0.0;

	for (k=0; k<IRL;k++)
	{
//		primi 4 filtri passa alto in cascata
		A = X[k] - ritardo1;
		ritardo1 = X[k] * coef1 + ritardo1 * coef1m;
		B = A - ritardo2;
		ritardo2 = A * coef2 + ritardo2 * coef2m;
		C = B - ritardo3;
		ritardo3 = B * coef3 + ritardo3 * coef3m;
		D = C - ritardo4;
		ritardo4 = C * coef4 + ritardo4 * coef4m;

//		2 filtri passa basso conclusivi
		E = D * coef5 + E * coef5m;
		F = E * coef6 + F * coef6m;
		X[k]=(float)(F/0.858); // correzione per rendere il guadagno ad 1kHz = 0 dB
	}
}


void BandFilter(float f0,float fQ,float  *fIRFilt,float fSamp,long IRL)
{   
	// filtro IIR a 2 poli, BandPass
	double PI=3.141592653589793;
	double Teta0 = 2.0 * PI * f0 / fSamp;
    double d = 2.0 * tan(Teta0 / 2.0 / fQ) / sin(Teta0);
	double Beta = 1.0/2.0*(1.0-0.5*d*sin(Teta0))/(1.0+0.5*d*sin(Teta0));
	double Gamma = (1.0 / 2.0 + Beta) * cos(Teta0);
//  BP
	double Mu = 0.0;
	double Alfa = (1.0/2.0-Beta)/2.0;
	double Sigma = -1.0;

	IIR(fIRFilt,IRL,Alfa,Beta,Gamma,Mu,Sigma);
}

void LPFilter(float f0,float fQ,float  *fIRFilt,float fSamp,long IRL)
{   
	// filtro IIR a 2 poli, LowPass
	double PI=3.141592653589793;
	double Teta0 = 2.0 * PI * f0 / fSamp;
    double d = 2.0 * tan(Teta0 / 2.0 / fQ) / sin(Teta0);
	double Beta = 1.0/2.0*(1.0-0.5*d*sin(Teta0))/(1.0+0.5*d*sin(Teta0));
	double Gamma = (1.0 / 2.0 + Beta) * cos(Teta0);
//  LP
	double Mu = 2.0;
	double Alfa = (0.5+Beta-Gamma)/4.0;
	double Sigma = 1.0;

	IIR(fIRFilt,IRL,Alfa,Beta,Gamma,Mu,Sigma);
}

void HPFilter(float f0,float fQ,float  *fIRFilt,float fSamp,long IRL)
{   
	// filtro IIR a 2 poli, HighPass
	double PI=3.141592653589793;
	double Teta0 = 2.0 * PI * f0 / fSamp;
    double d = 2.0 * tan(Teta0 / 2.0 / fQ) / sin(Teta0);
	double Beta = 1.0/2.0*(1.0-0.5*d*sin(Teta0))/(1.0+0.5*d*sin(Teta0));
	double Gamma = (1.0 / 2.0 + Beta) * cos(Teta0);
//  HP
	double Mu = -2.0;
	double Alfa = (0.5+Beta+Gamma)/4.0;
	double Sigma = 1.0;

	IIR(fIRFilt,IRL,Alfa,Beta,Gamma,Mu,Sigma);
}

void NotchFilter(float f0,float fQ,float  *fIRFilt,float fSamp,long IRL)
{   
	// filtro IIR a 2 poli, Notch
	double PI=3.141592653589793;
	double Teta0 = 2.0 * PI * f0 / fSamp;
    double d = 2.0 * tan(Teta0 / 2.0 / fQ) / sin(Teta0);
	double Beta = 1.0/2.0*(1.0-0.5*d*sin(Teta0))/(1.0+0.5*d*sin(Teta0));
	double Gamma = (1.0 / 2.0 + Beta) * cos(Teta0);
//  BR
	double Mu = -2 * cos(Teta0);
	double Alfa = (0.5+Beta)/2.0;
	double Sigma = 1.0;

	IIR(fIRFilt,IRL,Alfa,Beta,Gamma,Mu,Sigma);
}

void LFilter(float  *X,float fSamp,long IRL)
{   
	//filtro lineare con rimozione rumore sotto i 20 Hz (High Pass 6 poli)
	float f0,fQ; 
//  filtro 1 - passa alto 10 Hz
		f0=10.0f; // Hz
		fQ=0.707f;
		HPFilter(f0,fQ,X,fSamp,IRL);	
//  filtro 2 - passa alto 14 Hz
		f0=14.0f; // Hz
		fQ=0.707f;
		HPFilter(f0,fQ,X,fSamp,IRL);	
//  filtro 3 - passa alto 20 Hz
		f0=20.0f; // Hz
		fQ=0.707f;
		HPFilter(f0,fQ,X,fSamp,IRL);	
}

void IntegFilter(float  *X,long IRL)
{   
	//filtro integratore semplice
	long k;
	for (k=1; k<IRL;k++)
	{
		X[k]+=X[k-1];
	}
}


void BPFilter(float  *X,float fSamp,long IRL)
{   
	// Filtro passa-banda secondo ITU-P56 (160-5500 Hz)
	float f0,fQ; 
//  filtro 1 - passa alto 110 Hz
		f0=110.0f; // Hz
		fQ=0.7f; // Butterworth
		HPFilter(f0,fQ,X,fSamp,IRL);	
//  filtro 2 - passa alto 90 Hz
		f0=90.0f; // Hz
		fQ=1.5f;
		HPFilter(f0,fQ,X,fSamp,IRL);	
//  filtro 3 - passa basso 6500 Hz
		f0=6500.0f; // Hz
		fQ=0.8f;
		LPFilter(f0,fQ,X,fSamp,IRL);	
//  filtro 4 - passa basso 7500 Hz
		f0=7500.0f; // Hz
		fQ=1.8f;     //era 1.9
		LPFilter(f0,fQ,X,fSamp,IRL);	
//  filtro 5 - passa banda 1100 Hz
		f0=1100.0f; // Hz
		fQ=0.11f;
		BandFilter(f0,fQ,X,fSamp,IRL);
// Correzione guadagno -0.11 dB
		Gain(-0.15f,X,IRL);
}

void Lp250Filter(float  *X,float fSamp,long IRL)
{   
	// Filtro passa-banda secondo ITU-P56 (160-5500 Hz)
	float f0,fQ; 
//  filtro 1 - passa basso 250 Hz
		f0=320.0f; // Hz
		fQ=1.0f; // Butterworth
		LPFilter(f0,fQ,X,fSamp,IRL);	
//  filtro 2 - passa basso 260 Hz
		f0=360.0f; // Hz
		fQ=0.6f; // Butterworth
		LPFilter(f0,fQ,X,fSamp,IRL);	
//  filtro 3 - passa basso 270 Hz
		f0=400.0f; // Hz
		fQ=1.1f; // Butterworth
		LPFilter(f0,fQ,X,fSamp,IRL);	
//  filtro 4 - passa basso 280 Hz
		f0=500.0f; // Hz
		fQ=0.5f; // Butterworth
		LPFilter(f0,fQ,X,fSamp,IRL);	
// Correzione guadagno 0.0 dB
		Gain(0.0f,X,IRL);
}

void Vel1Filter(float  *X,float fSamp,long IRL)
{   
	// Filtro Integratore piu' HP ad 1 Hz
	float f0,fQ;
	float G;
//  filtro integratore
	    IntegFilter(X,IRL);
//  filtro 1 - passa alto 1 Hz
		f0=1.0f; // Hz
		fQ=1.0f; // Butterworth
		HPFilter(f0,fQ,X,fSamp,IRL);	
// Correzione guadagno in funzione di Fsamp in modo da mantenerlo invariato a 159.15 Hz
		G=(float)(-33.6-20.0*log(fSamp/48000.0)/log(10.0));
		Gain(G,X,IRL);
}